'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueClassComponent = require('vue-class-component');

function createDecorator(factory) {
    return function (target, key, index) {
        var Ctor = typeof target === 'function'
            ? target
            : target.constructor;
        if (!Ctor.__d) {
            Ctor.__d = [];
            Ctor.__d.__n = Ctor.name;
        }
        else if (Ctor.__d.__n !== Ctor.name) {
            Ctor.__d = [].concat(Ctor.__d);
            Ctor.__d.__n = Ctor.name;
        }
        if (typeof index !== 'number') {
            index = undefined;
        }
        Ctor.__d.push(function (options) { return factory(options, key, index); });
    };
}

function State(target, propertyKey) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (typeof target === 'string') {
        var value = target;
        var arr_1 = value ? value.split("/") : undefined;
        return function (target, propertyKey) {
            delete target[propertyKey];
            createDecorator(function (options, key) {
                options.computed[key] = function () {
                    var data = this.$store.state;
                    if (!arr_1) {
                        return data[key];
                    }
                    else {
                        arr_1.forEach(function (item) {
                            data = data[item];
                        });
                        return data;
                    }
                };
            })(target, propertyKey);
        };
    }
    else {
        delete target[propertyKey];
        createDecorator(function (options, key) {
            options.computed[key] = function () {
                var data = this.$store.state;
                return data[key];
            };
        })(target, propertyKey);
    }
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

function Action(target, propertyKey) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (typeof target === 'string') {
        var value_1 = target;
        return function (target, propertyKey) {
            delete target[propertyKey];
            createDecorator(function (options, key) {
                var name = value_1 || key;
                options.methods[key] = function () {
                    var _a;
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return (_a = this.$store).dispatch.apply(_a, __spreadArray([name], args));
                };
            })(target, propertyKey);
        };
    }
    else {
        delete target[propertyKey];
        createDecorator(function (options, key) {
            var name = key;
            options.methods[key] = function () {
                var _a;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return (_a = this.$store).dispatch.apply(_a, __spreadArray([name], args));
            };
        })(target, propertyKey);
    }
}

function Getter(target, propertyKey) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    var arg1Type = typeof target;
    if (arg1Type === 'string') {
        var value_1 = target;
        return function (target, propertyKey) {
            delete target[propertyKey];
            createDecorator(function (options, key) {
                var name = value_1 || key;
                options.computed[key] = function () {
                    var getter = this.$store.getters[name];
                    return getter;
                };
            })(target, propertyKey);
        };
    }
    else {
        delete target[propertyKey];
        createDecorator(function (options, key) {
            var name = key;
            options.computed[name] = function () {
                var getter = this.$store.getters[name];
                return getter;
            };
        })(target, propertyKey);
    }
}

function createMutation(name, target, propertyKey, descriptor) {
    createDecorator(function (options, key) {
        var dname = name || key;
        options.methods[key] = function () {
            var _a;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return (_a = this.$store).commit.apply(_a, __spreadArray([dname], args));
        };
    })(target, propertyKey);
}
function Mutation(target, propertyKey, descriptor) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (typeof target === 'string') {
        var name_1 = target;
        return function (target, propertyKey, descriptor) {
            createMutation(name_1, target, propertyKey);
        };
    }
    else {
        var name_2 = propertyKey;
        createMutation(name_2, target, propertyKey);
    }
}

function Watch(target, propertyKey, descriptor) {
    if (propertyKey === void 0) { propertyKey = ""; }
    if (typeof target === 'string') {
        var value_1 = target;
        return function (target, propertyKey, descriptor) {
            delete target[propertyKey];
            createDecorator(function (options, key) {
                if (!options.watch) {
                    options.watch = {};
                }
                var name = value_1 || key;
                var watch = options.watch;
                watch[name] = descriptor.value;
            })(target, propertyKey);
        };
    }
    else {
        delete target[propertyKey];
        createDecorator(function (options, key) {
            if (!options.watch) {
                options.watch = {};
            }
            var name = key;
            var watch = options.watch;
            watch[name] = descriptor === null || descriptor === void 0 ? void 0 : descriptor.value;
        })(target, propertyKey);
    }
}

function createProvide(name, target, propertyKey, descriptor) {
    createDecorator(function (options, key) {
        if (!options.provide) {
            var provideFun_1 = function () {
                var _this = this;
                var result = Object.create(null);
                Object.entries(provideFun_1.values).forEach(function (item) {
                    var name = item[0], value = item[1];
                    result[name] = _this[value];
                });
                Object.entries(provideFun_1.funs).forEach(function (item) {
                    var name = item[0], value = item[1];
                    result[name] = function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        return _this[value].apply(_this, args);
                    };
                });
                return result;
            };
            provideFun_1.values = {};
            provideFun_1.funs = {};
            options.provide = provideFun_1;
        }
        var dname = name || key;
        var provide = options.provide;
        if (descriptor) {
            provide.funs[dname] = key;
        }
        else {
            provide.values[dname] = key;
        }
    })(target, propertyKey);
}
function Provide(target, propertyKey, descriptor) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (propertyKey) {
        createProvide(propertyKey, target, propertyKey, descriptor);
    }
    else {
        var name_1 = target;
        return function (target, propertyKey, descriptor) {
            createProvide(name_1, target, propertyKey, descriptor);
        };
    }
}

function createInject(params, target, propertyKey, descriptor) {
    createDecorator(function (options, key, index) {
        var data = Object.assign({ from: key }, params);
        if (descriptor && typeof data["default"] === 'undefined') {
            data["default"] = function () { return descriptor.value; };
            if (options.methods) {
                delete options.methods[key];
            }
        }
        if (!options.inject) {
            options.inject = {};
        }
        var inject = options.inject;
        inject[key] = data;
    })(target, propertyKey);
}
function Inject(target, propertyKey, descriptor) {
    if (target === void 0) { target = ""; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (propertyKey) {
        createInject({ from: propertyKey }, target, propertyKey, descriptor);
    }
    else {
        var params_1 = typeof target === 'string' ? { from: target } : target;
        return function (target, propertyKey, descriptor) {
            createInject(params_1, target, propertyKey, descriptor);
        };
    }
}

function createProp(params, target, propertyKey, descriptor) {
    createDecorator(function (options, key) {
        if (!options.props) {
            options.props = {};
        }
        var props = options.props;
        if (descriptor && typeof params["default"] === 'undefined') {
            params["default"] = function () { return descriptor.value; };
            if (options.methods) {
                delete options.methods[key];
            }
        }
        props[key] = params;
    })(target, propertyKey);
}
function Prop(target, propertyKey, descriptor) {
    if (target === void 0) { target = {}; }
    if (propertyKey === void 0) { propertyKey = ""; }
    if (propertyKey) {
        createProp({}, target, propertyKey, descriptor);
    }
    else {
        var params_1 = target;
        return function (target, propertyKey, descriptor) {
            createProp(params_1, target, propertyKey, descriptor);
        };
    }
}

function initOptions(options) {
    if (!options.emits) {
        options.emits = {};
    }
    else if (Array.isArray(options.emits)) {
        var map_1 = {};
        options.emits.forEach(function (item) {
            map_1[item] = null;
        });
        options.emits = map_1;
    }
}
function appendValues(options, value) {
    initOptions(options);
    var emits = options.emits;
    var valueType = typeof (value);
    if (valueType === 'function') {
        value = value();
    }
    valueType = typeof (value);
    if (Array.isArray(value)) {
        value.forEach(function (item) {
            emits[item] = null;
        });
    }
    else if (valueType === 'object') {
        Object.assign(emits, value);
    }
    else if (valueType === 'string') {
        emits[value] = null;
    }
}
function Emits(target, propertyKey, descriptor) {
    if (propertyKey === void 0) { propertyKey = ""; }
    if (!propertyKey) {
        var value_1 = target;
        return function (target, propertyKey, descriptor) {
            delete target[propertyKey];
            createDecorator(function (options, key) {
                appendValues(options, value_1);
            })(target, propertyKey);
        };
    }
    else {
        delete target[propertyKey];
        createDecorator(function (options, key) {
            var value = descriptor === null || descriptor === void 0 ? void 0 : descriptor.value;
            appendValues(options, value);
        })(target, propertyKey);
    }
}

Object.defineProperty(exports, 'Options', {
  enumerable: true,
  get: function () {
    return vueClassComponent.Options;
  }
});
Object.defineProperty(exports, 'mixins', {
  enumerable: true,
  get: function () {
    return vueClassComponent.mixins;
  }
});
exports.Action = Action;
exports.Emits = Emits;
exports.Getter = Getter;
exports.Inject = Inject;
exports.Mutation = Mutation;
exports.Prop = Prop;
exports.Provide = Provide;
exports.State = State;
exports.Watch = Watch;
